@@ -28,185 +28,151 @@ options:
   auto_remove:
     description:
       - enable auto-removal of the container on daemon side when the container's process exits
-    default: false
+    type: bool
+    default: 'no'
     version_added: "2.4"
   blkio_weight:
     description:
       - Block IO (relative weight), between 10 and 1000.
-    default: null
-    required: false
   capabilities:
     description:
       - List of capabilities to add to the container.
-    default: null
-    required: false
   cleanup:
     description:
       - Use with I(detach=false) to remove the container after successful execution.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
     version_added: "2.2"
   command:
     description:
       - Command to execute when the container starts.
         A command may be either a string or a list.
         Prior to version 2.4, strings were split on commas.
-    default: null
-    required: false
   cpu_period:
     description:
       - Limit CPU CFS (Completely Fair Scheduler) period
     default: 0
-    required: false
   cpu_quota:
     description:
       - Limit CPU CFS (Completely Fair Scheduler) quota
     default: 0
-    required: false
   cpuset_cpus:
     description:
       - CPUs in which to allow execution C(1,3) or C(1-3).
-    default: null
-    required: false
   cpuset_mems:
     description:
       - Memory nodes (MEMs) in which to allow execution C(0-3) or C(0,1)
-    default: null
-    required: false
   cpu_shares:
     description:
       - CPU shares (relative weight).
-    default: null
-    required: false
   detach:
     description:
       - Enable detached mode to leave the container running in background.
         If disabled, the task will reflect the status of the container run (failed if the command failed).
+    type: bool
     default: true
-    required: false
   devices:
     description:
       - "List of host device bindings to add to the container. Each binding is a mapping expressed
         in the format: <path_on_host>:<path_in_container>:<cgroup_permissions>"
-    default: null
-    required: false
+  dns_opts:
+    description:
+      - list of DNS options
   dns_servers:
     description:
       - List of custom DNS servers.
-    default: null
-    required: false
   dns_search_domains:
     description:
       - List of custom DNS search domains.
-    default: null
-    required: false
   domainname:
     description:
       - Container domainname.
-    default: null
-    required: false
     version_added: "2.5"
   env:
     description:
       - Dictionary of key,value pairs.
-    default: null
-    required: false
   env_file:
     version_added: "2.2"
     description:
       - Path to a file containing environment variables I(FOO=BAR).
       - If variable also present in C(env), then C(env) value will override.
-      - Requires docker-py >= 1.4.0.
-    default: null
-    required: false
   entrypoint:
     description:
       - Command that overwrites the default ENTRYPOINT of the image.
-    default: null
-    required: false
   etc_hosts:
     description:
       - Dict of host-to-IP mappings, where each host name is a key in the dictionary.
         Each host name will be added to the container's /etc/hosts file.
-    default: null
-    required: false
   exposed_ports:
     description:
       - List of additional container ports which informs Docker that the container
         listens on the specified network ports at runtime.
         If the port is already exposed using EXPOSE in a Dockerfile, it does not
         need to be exposed again.
-    default: null
-    required: false
     aliases:
       - exposed
+      - expose
   force_kill:
     description:
       - Use the kill command when stopping a running container.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
+    aliases:
+      - forcekill
   groups:
     description:
       - List of additional group names and/or IDs that the container process will run as.
-    default: null
-    required: false
   hostname:
     description:
       - Container hostname.
-    default: null
-    required: false
   ignore_image:
     description:
       - When C(state) is I(present) or I(started) the module compares the configuration of an existing
         container to requested configuration. The evaluation includes the image version. If
         the image version in the registry does not match the container, the container will be
         recreated. Stop this behavior by setting C(ignore_image) to I(True).
-    default: false
-    required: false
+    type: bool
+    default: 'no'
     version_added: "2.2"
   image:
     description:
       - Repository path and tag used to create the container. If an image is not found or pull is true, the image
         will be pulled from the registry. If no tag is included, 'latest' will be used.
-    default: null
-    required: false
+  init:
+    description:
+      - Run an init inside the container that forwards signals and reaps processes.
+        This option requires Docker API 1.25+.
+    type: bool
+    default: 'no'
+    version_added: "2.6"
   interactive:
     description:
       - Keep stdin open after a container is launched, even if not attached.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   ipc_mode:
     description:
       - Set the IPC mode for the container. Can be one of 'container:<name|id>' to reuse another
         container's IPC namespace or 'host' to use the host's IPC namespace within the container.
-    default: null
-    required: false
   keep_volumes:
     description:
       - Retain volumes associated with a removed container.
-    default: true
-    required: false
+    type: bool
+    default: 'yes'
   kill_signal:
     description:
       - Override default signal used to kill a running container.
-    default: null
-    required: false
   kernel_memory:
     description:
       - "Kernel memory limit (format: <number>[<unit>]). Number is a positive integer.
         Unit can be one of b, k, m, or g. Minimum is 4M."
     default: 0
-    required: false
   labels:
      description:
        - Dictionary of key value pairs.
-     default: null
-     required: false
   links:
     description:
       - List of name aliases for linked containers in the format C(container_name:alias)
-    default: null
-    required: false
   log_driver:
     description:
       - Specify the logging driver. Docker uses json-file by default.
@@ -219,42 +185,34 @@ options:
       - fluentd
       - awslogs
       - splunk
-    default: null
-    required: false
   log_options:
     description:
       - Dictionary of options specific to the chosen log_driver. See https://docs.docker.com/engine/admin/logging/overview/
         for details.
-    required: false
-    default: null
+    aliases:
+      - log_opt
   mac_address:
     description:
       - Container MAC address (e.g. 92:d0:c6:0a:29:33)
-    default: null
-    required: false
   memory:
     description:
       - "Memory limit (format: <number>[<unit>]). Number is a positive integer.
         Unit can be one of b, k, m, or g"
-    default: 0
-    required: false
+    default: '0'
   memory_reservation:
     description:
       - "Memory soft limit (format: <number>[<unit>]). Number is a positive integer.
         Unit can be one of b, k, m, or g"
     default: 0
-    required: false
   memory_swap:
     description:
       - Total memory limit (memory + swap, format:<number>[<unit>]).
         Number is a positive integer. Unit can be one of b, k, m, or g.
     default: 0
-    required: false
   memory_swappiness:
     description:
         - Tune a container's memory swappiness behavior. Accepts an integer between 0 and 100.
     default: 0
-    required: false
   name:
     description:
       - Assign a name to a new container or match an existing container.
@@ -262,19 +220,10 @@ options:
     required: true
   network_mode:
     description:
-      - Connect the container to a network.
-    choices:
-      - bridge
-      - container:<name|id>
-      - host
-      - none
-    default: null
-    required: false
+      - Connect the container to a network. Choices are "bridge", "host", "none" or "container:<name|id>"
   userns_mode:
      description:
        - User namespace to use
-     default: null
-     required: false
      version_added: "2.5"
   networks:
      description:
@@ -284,35 +233,30 @@ options:
        - If included, C(links) or C(aliases) are lists.
        - For examples of the data structure and usage see EXAMPLES below.
        - To remove a container from one or more networks, use the C(purge_networks) option.
-     default: null
-     required: false
      version_added: "2.2"
   oom_killer:
     description:
       - Whether or not to disable OOM Killer for the container.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   oom_score_adj:
     description:
       - An integer value containing the score given to the container in order to tune OOM killer preferences.
     default: 0
-    required: false
     version_added: "2.2"
   paused:
     description:
       - Use with the started state to pause running processes inside the container.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   pid_mode:
     description:
       - Set the PID namespace mode for the container. Currently only supports 'host'.
-    default: null
-    required: false
   privileged:
     description:
       - Give extended privileges to the container.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   published_ports:
     description:
       - List of ports to publish from the container to the host.
@@ -329,62 +273,53 @@ options:
         value encountered in the list of C(networks) is the one that will be used.
     aliases:
       - ports
-    required: false
-    default: null
   pull:
     description:
        - If true, always pull the latest version of an image. Otherwise, will only pull an image when missing.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   purge_networks:
     description:
        - Remove the container from ALL networks not included in C(networks) parameter.
        - Any default networks such as I(bridge), if not found in C(networks), will be removed as well.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
     version_added: "2.2"
   read_only:
     description:
       - Mount the container's root file system as read-only.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   recreate:
     description:
       - Use with present and started states to force the re-creation of an existing container.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   restart:
     description:
       - Use with started state to force a matching container to be stopped and restarted.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   restart_policy:
     description:
       - Container restart policy. Place quotes around I(no) option.
     choices:
-      - always
-      - no
-      - on-failure
-      - unless-stopped
-    default: on-failure
-    required: false
+      - 'no'
+      - 'on-failure'
+      - 'always'
+      - 'unless-stopped'
   restart_retries:
     description:
        - Use with restart policy to control maximum number of restart attempts.
     default: 0
-    required: false
   shm_size:
     description:
       - Size of `/dev/shm`. The format is `<number><unit>`. `number` must be greater than `0`.
         Unit is optional and can be `b` (bytes), `k` (kilobytes), `m` (megabytes), or `g` (gigabytes).
       - Omitting the unit defaults to bytes. If you omit the size entirely, the system uses `64m`.
-    default: null
-    required: false
   security_opts:
     description:
       - List of security options in the form of C("label:user:User")
-    default: null
-    required: false
   state:
     description:
       - 'I(absent) - A container matching the specified name will be stopped and removed. Use force_kill to kill the container
@@ -406,7 +341,6 @@ options:
         with a removed container.'
       - 'I(stopped) - Asserts that the container is first I(present), and then if the container is running moves it to a stopped
         state. Use force_kill to kill a container rather than stopping it.'
-    required: false
     default: started
     choices:
       - absent
@@ -416,51 +350,37 @@ options:
   stop_signal:
     description:
       - Override default signal used to stop the container.
-    default: null
-    required: false
   stop_timeout:
     description:
       - Number of seconds to wait for the container to stop before sending SIGKILL.
-    required: false
-    default: null
   trust_image_content:
     description:
-      - If true, skip image verification.
-    default: false
-    required: false
+      - If C(yes), skip image verification.
+    type: bool
+    default: 'no'
   tmpfs:
     description:
       - Mount a tmpfs directory
-    default: null
-    required: false
     version_added: 2.4
   tty:
     description:
       - Allocate a pseudo-TTY.
-    default: false
-    required: false
+    type: bool
+    default: 'no'
   ulimits:
     description:
       - "List of ulimit options. A ulimit is specified as C(nofile:262144:262144)"
-    default: null
-    required: false
   sysctls:
     description:
       - Dictionary of key,value pairs.
-    default: null
-    required: false
     version_added: 2.4
   user:
     description:
       - Sets the username or UID used and optionally the groupname or GID for the specified command.
       - "Can be [ user | user:group | uid | uid:gid | user:gid | uid:group ]"
-    default: null
-    required: false
   uts:
     description:
       - Set the UTS namespace mode for the container.
-    default: null
-    required: false
   volumes:
     description:
       - List of volumes to mount within the container.
@@ -468,23 +388,15 @@ options:
       - You can specify a read mode for the mount with either C(ro) or C(rw).
       - SELinux hosts can additionally use C(z) or C(Z) to use a shared or
         private label for the volume.
-    default: null
-    required: false
   volume_driver:
     description:
       - The container volume driver.
-    default: none
-    required: false
   volumes_from:
     description:
       - List of container names or Ids to get volumes from.
-    default: null
-    required: false
   working_dir:
     description:
       - Path to the working directory.
-    default: null
-    required: false
     version_added: "2.4"
 extends_documentation_fragment:
     - docker
@@ -496,13 +408,18 @@ author:
     - "Thomas Steinbach (@ThomasSteinbach)"
     - "Philippe Jandot (@zfil)"
     - "Daan Oosterveld (@dusdanig)"
-    - "James Tanner (@jctanner)"
     - "Chris Houseknecht (@chouseknecht)"
     - "Kassian Sun (@kassiansun)"
 
 requirements:
     - "python >= 2.6"
     - "docker-py >= 1.7.0"
+    - "Please note that the L(docker-py,https://pypi.org/project/docker-py/) Python
+       module has been superseded by L(docker,https://pypi.org/project/docker/)
+       (see L(here,https://github.com/docker/docker-py/issues/1310) for details).
+       For Python 2.6, C(docker-py) must be used. Otherwise, it is recommended to
+       install the C(docker) Python module. Note that both modules should I(not)
+       be installed at the same time."
     - "Docker API >= 1.20"
 '''
 
@@ -686,6 +603,7 @@ docker_container:
 import os
 import re
 import shlex
+from distutils.version import LooseVersion
 
 from ansible.module_utils.basic import human_to_bytes
 from ansible.module_utils.docker_common import HAS_DOCKER_PY_2, HAS_DOCKER_PY_3, AnsibleDockerClient, DockerBaseClass
@@ -697,6 +615,7 @@ try:
         from docker.types import Ulimit, LogConfig
     else:
         from docker.utils.types import Ulimit, LogConfig
+    from ansible.module_utils.docker_common import docker_version
 except:
     # missing docker-py handled in ansible.module_utils.docker
     pass
@@ -748,6 +667,7 @@ class TaskParameters(DockerBaseClass):
         self.hostname = None
         self.ignore_image = None
         self.image = None
+        self.init = None
         self.interactive = None
         self.ipc_mode = None
         self.keep_volumes = None
@@ -842,6 +762,10 @@ class TaskParameters(DockerBaseClass):
                 if network.get('links'):
                     network['links'] = self._parse_links(network['links'])
 
+        if self.mac_address:
+            # Ensure the MAC address uses colons instead of hyphens for later comparison
+            self.mac_address = self.mac_address.replace('-', ':')
+
         if self.entrypoint:
             # convert from list to str.
             self.entrypoint = ' '.join([str(x) for x in self.entrypoint])
@@ -989,10 +913,10 @@ class TaskParameters(DockerBaseClass):
             group_add='groups',
             devices='devices',
             pid_mode='pid_mode',
-            tmpfs='tmpfs'
+            tmpfs='tmpfs',
         )
 
-        if HAS_DOCKER_PY_2 or HAS_DOCKER_PY_3:
+        if self.client.HAS_AUTO_REMOVE_OPT:
             # auto_remove is only supported in docker>=2
             host_config_params['auto_remove'] = 'auto_remove'
 
@@ -1001,6 +925,9 @@ class TaskParameters(DockerBaseClass):
             host_config_params['cpu_shares'] = 'cpu_shares'
             host_config_params['volume_driver'] = 'volume_driver'
 
+        if self.client.HAS_INIT_OPT:
+            host_config_params['init'] = 'init'
+
         params = dict()
         for key, value in host_config_params.items():
             if getattr(self, value, None) is not None:
@@ -2062,6 +1989,27 @@ class ContainerManager(DockerBaseClass):
         return response
 
 
+class AnsibleDockerClientContainer(AnsibleDockerClient):
+
+    def __init__(self, **kwargs):
+        super(AnsibleDockerClientContainer, self).__init__(**kwargs)
+
+        docker_api_version = self.version()['ApiVersion']
+        init_supported = LooseVersion(docker_api_version) >= LooseVersion('1.25')
+        if self.module.params.get("init") and not init_supported:
+            self.fail('docker API version is %s. Minimum version required is 1.25 to set init option.' % (docker_api_version,))
+
+        init_supported = init_supported and LooseVersion(docker_version) >= LooseVersion('2.2')
+        if self.module.params.get("init") and not init_supported:
+            self.fail("docker or docker-py version is %s. Minimum version required is 2.2 to set init option. "
+                      "If you use the 'docker-py' module, you have to switch to the docker 'Python' package." % (docker_version,))
+
+        self.HAS_INIT_OPT = init_supported
+        self.HAS_AUTO_REMOVE_OPT = HAS_DOCKER_PY_2 or HAS_DOCKER_PY_3
+        if self.module.params.get('auto_remove') and not self.HAS_AUTO_REMOVE_OPT:
+            self.fail("'auto_remove' is not compatible with the 'docker-py' Python package. It requires the newer 'docker' Python package.")
+
+
 def main():
     argument_spec = dict(
         auto_remove=dict(type='bool', default=False),
@@ -2090,6 +2038,7 @@ def main():
         hostname=dict(type='str'),
         ignore_image=dict(type='bool', default=False),
         image=dict(type='str'),
+        init=dict(type='bool', default=False),
         interactive=dict(type='bool', default=False),
         ipc_mode=dict(type='str'),
         keep_volumes=dict(type='bool', default=True),
@@ -2145,15 +2094,12 @@ def main():
         ('state', 'present', ['image'])
     ]
 
-    client = AnsibleDockerClient(
+    client = AnsibleDockerClientContainer(
         argument_spec=argument_spec,
         required_if=required_if,
         supports_check_mode=True
     )
 
-    if (not (HAS_DOCKER_PY_2 or HAS_DOCKER_PY_3)) and client.module.params.get('auto_remove'):
-        client.module.fail_json(msg="'auto_remove' is not compatible with the 'docker-py' Python package. It requires the newer 'docker' Python package.")
-
     cm = ContainerManager(client)
     client.module.exit_json(**cm.results)
 
