@@ -32,13 +32,10 @@ options:
     description:
       - Full name of interface, i.e. Ethernet1/1, port-channel10.
     required: true
-    default: null
     aliases: [interface]
   interface_type:
     description:
       - Interface type to be unconfigured from the device.
-    required: false
-    default: null
     choices: ['loopback', 'portchannel', 'svi', 'nve']
     version_added: 2.2
   speed:
@@ -48,42 +45,32 @@ options:
   admin_state:
     description:
       - Administrative state of the interface.
-    required: false
     default: up
     choices: ['up','down']
   description:
     description:
       - Interface description.
-    required: false
-    default: null
   mode:
     description:
       - Manage Layer 2 or Layer 3 state of the interface.
         This option is supported for ethernet and portchannel interface.
         Applicable for ethernet and portchannel interface only.
-    required: false
-    default: null
     choices: ['layer2','layer3']
   mtu:
     description:
       - MTU for a specific interface. Must be an even number between 576 and 9216.
         Applicable for ethernet interface only.
-    required: false
     version_added: 2.5
   ip_forward:
     description:
       - Enable/Disable ip forward feature on SVIs.
-    required: false
-    default: null
     choices: ['enable','disable']
     version_added: 2.2
   fabric_forwarding_anycast_gateway:
     description:
       - Associate SVI with anycast gateway under VLAN configuration mode.
         Applicable for SVI interface only.
-    required: false
-    default: null
-    choices: ['true','false']
+    type: bool
     version_added: 2.2
   duplex:
     description:
@@ -121,7 +108,6 @@ options:
   state:
     description:
       - Specify desired state of the resource.
-    required: true
     default: present
     choices: ['present','absent','default']
   delay:
@@ -281,7 +267,7 @@ def get_vlan_interface_attributes(name, intf_type, module):
     command = 'show run interface {0} all'.format(name)
     try:
         body = execute_show_command(command, module)[0]
-    except IndexError:
+    except (IndexError, TypeError):
         return None
     if body:
         command_list = body.split('\n')
@@ -518,7 +504,7 @@ def map_config_to_obj(want, module):
         if body:
             try:
                 interface_table = body['TABLE_interface']['ROW_interface']
-            except KeyError:
+            except (KeyError, TypeError):
                 return list()
 
             if interface_table:
@@ -613,11 +599,12 @@ def check_declarative_intent_params(module, want):
             return
 
         cmd = [{'command': 'show interface {0}'.format(w['name']), 'output': 'text'}]
-        output = run_commands(module, cmd, check_rc=False)
-        if output:
-            out = output[0]
-        else:
+
+        try:
+            out = run_commands(module, cmd, check_rc=False)[0]
+        except (AttributeError, IndexError, TypeError):
             out = ''
+
         if want_tx_rate:
             match = re.search(r'output rate (\d+)', out, re.M)
             have_tx_rate = None
