@@ -12,6 +12,12 @@ test="$1"
 docker images ansible/ansible
 docker ps
 
+for container in $(docker ps --format '{{.Image}} {{.ID}}' | grep -v '^drydock/' | sed 's/^.* //'); do
+    docker rm -f "${container}"
+done
+
+docker ps
+
 if [ -d /home/shippable/cache/ ]; then
     ls -la /home/shippable/cache/
 fi
